/* PPLite: a lightweight library for convex polyhedra derived from PPL.
   Copyright (C) 2018-2020 Enea Zaffanella <enea.zaffanella@unipr.it>

This file is part of PPLite.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "pplite_test.hh"

bool
test01() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A <= 4);
  cs.push_back(B <= 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(A >= 4);

  Poly ph2 = ph1.split(c, Topol::CLOSED);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(A >= 4);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(A <= 4);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test02() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A <= 4);
  cs.push_back(B <= 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(A >= 2);

  Poly ph2 = ph1.split(c, Topol::CLOSED);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(A >= 2);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(A <= 2);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test03() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A <= 4);
  cs.push_back(B <= 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(A <= 0);

  Poly ph2 = ph1.split(c, Topol::CLOSED);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(A <= 0);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(A >= 0);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test04() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A <= 4);
  cs.push_back(B <= 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(A <= 2);

  Poly ph2 = ph1.split(c, Topol::CLOSED);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(A <= 2);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(A >= 2);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test05() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B > 0);
  cs.push_back(A <= 4);
  cs.push_back(B < 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(B <= 2);

  Poly ph2 = ph1.split(c, Topol::CLOSED);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(B <= 2);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(B >= 2);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test06() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A > 0);
  cs.push_back(B > 0);
  cs.push_back(A < 4);
  cs.push_back(B < 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(B <= 2);

  Poly ph2 = ph1.split(c, Topol::CLOSED);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(B <= 2);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(B >= 2);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test07() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A < 4);
  cs.push_back(B <= 4);
  cs.push_back(A + B > 0);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(B >= 2);

  Poly ph2 = ph1.split(c, Topol::CLOSED);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(B >= 2);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(B <= 2);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test08() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A < 4);
  cs.push_back(B < 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(A + B <= 4);

  Poly ph2 = ph1.split(c, Topol::CLOSED);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(A + B <= 4);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(A + B >= 4);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test09() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A < 4);
  cs.push_back(B < 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(A + B <= 0);

  Poly ph2 = ph1.split(c, Topol::CLOSED);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(A + B <= 0);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(A + B >= 0);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test10() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A < 4);
  cs.push_back(B < 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(A + B >= 4);

  Poly ph2 = ph1.split(c, Topol::CLOSED);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(A + B >= 4);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(A + B <= 4);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test11() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A < 4);
  cs.push_back(B < 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(A <= 4);

  Poly ph2 = ph1.split(c, Topol::CLOSED);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(A <= 4);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(A >= 4);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test12() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(A < 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(B <= 4);

  Poly ph2 = ph1.split(c, Topol::CLOSED);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(B <= 4);

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(B >= 4);

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test13() {
  Var A(0);
  Var B(1);

  Poly ph1(2, Topol::NNC);

  Con c(A + B <= 4);

  Poly ph2 = ph1.split(c, Topol::CLOSED);

  Poly knr1(2, Topol::NNC);
  knr1.add_con(A + B <= 4);

  Poly knr2(2, Topol::NNC);
  knr2.add_con(A + B >= 4);

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test14() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A > 0);
  cs.push_back(A < 3);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(A <= 2);

  Poly ph2 = ph1.split(c, Topol::CLOSED);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(A <= 2);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(A >= 2);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test15() {
  Var A(0);
  Var B(1);
  Var C(2);

  Gens gs;
  gs.push_back(closure_point());
  gs.push_back(point(A));
  gs.push_back(closure_point(B));
  gs.push_back(closure_point(C));
  gs.push_back(closure_point(A + B));
  gs.push_back(closure_point(A + C));
  gs.push_back(point(B + C));
  gs.push_back(closure_point(A + B + C));

  Poly ph1(3, Topol::NNC, Spec_Elem::EMPTY);
  ph1.add_gens(gs);
  ph1.minimize();

  Con c(A - B >= 0);

  Poly ph2 = ph1.split(c, Topol::CLOSED);

  Poly knr1(3, Topol::NNC, Spec_Elem::EMPTY);
  knr1.add_gens(gs);
  knr1.add_con(c);
  knr1.minimize();

  Poly knr2(3, Topol::NNC, Spec_Elem::EMPTY);
  knr2.add_gens(gs);
  knr2.add_con(A - B <= 0);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test16() {
  Cons cs;
  for (dim_type i = 0; i < 4; ++i) {
    if (i%2) {
      cs.push_back(Var(i) > 0);
      cs.push_back(Var(i) < 10);
    } else {
      cs.push_back(Var(i) >= 0);
      cs.push_back(Var(i) <= 10);
    }
  }
  cs.push_back(Var(0) >= 9);

  Poly ph1(4, Topol::NNC);
  ph1.add_cons(cs);
  ph1.minimize();

  Con c(Var(0) <= 10);

  Poly ph2 = ph1.split(c, Topol::CLOSED);

  Poly knr1(4, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(c);
  knr1.minimize();

  Poly knr2(4, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(Var(0) >= 10);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test17() {
  Cons cs;
  for (dim_type i = 0; i < 4; ++i) {
    if (i%2) {
      cs.push_back(Var(i) > 0);
      cs.push_back(Var(i) < 10);
    } else {
      cs.push_back(Var(i) >= 0);
      cs.push_back(Var(i) <= 10);
    }
  }

  Poly ph1(6, Topol::NNC);
  ph1.add_cons(cs);
  ph1.minimize();

  Con c(Var(0) <= 10);

  Poly ph2 = ph1.split(c, Topol::CLOSED);

  Poly knr1(6, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(c);

  Poly knr2(6, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(Var(0) >= 10);

  return (ph1 == knr1 && ph2 == knr2);
}

BEGIN_MAIN
  DO_TEST(test01);
  DO_TEST(test02);
  DO_TEST(test03);
  DO_TEST(test04);
  DO_TEST(test05);
  DO_TEST(test06);
  DO_TEST(test07);
  DO_TEST(test08);
  DO_TEST(test09);
  DO_TEST(test10);
  DO_TEST(test11);
  DO_TEST(test12);
  DO_TEST(test13);
  DO_TEST(test14);
  DO_TEST(test15);
  DO_TEST(test16);
  DO_TEST(test17);
END_MAIN

